@@ -1,12 +1,23 @@
 """Classes for representing entities beloging to an Imou device."""
+import asyncio
 import logging
 from abc import ABC, abstractmethod
 from datetime import datetime
 from typing import Optional, Union
 
 from .api import ImouAPIClient
-from .const import BINARY_SENSORS, BUTTONS, CAMERAS, IMOU_SWITCHES, SELECT, SENSORS, SIRENS
-from .exceptions import APIError, InvalidResponse
+from .const import (
+    BINARY_SENSORS,
+    BUTTONS,
+    CAMERA_WAIT_BEFORE_DOWNLOAD,
+    CAMERAS,
+    IMOU_SWITCHES,
+    ONLINE_STATUS,
+    SELECT,
+    SENSORS,
+    SIRENS,
+)
+from .exceptions import APIError, InvalidResponse, NotConnected
 
 _LOGGER: logging.Logger = logging.getLogger(__package__)
 
@@ -65,6 +76,19 @@ class ImouEntity(ABC):
         """Entity attributes."""
         return self._attributes
 
+    async def _async_is_ready(self) -> bool:
+        """Check if the sensor is fully ready."""
+        # check if the sensor is enabled
+        if not self._enabled:
+            return False
+        # wake up the device if a dormant device and sleeping
+        if self._device_instance is not None:
+            awake = await self._device_instance.async_wakeup()
+            if awake:
+                return True
+            return False
+        return True
+
     @abstractmethod
     async def async_update(self, **kwargs):
         """Update the entity."""
@@ -95,7 +119,7 @@ class ImouSensor(ImouEntity):
 
     async def async_update(self, **kwargs):
         """Update the entity."""
-        if not self._enabled:
+        if not await self._async_is_ready():
             return
 
         # storageUsed sensor
@@ -120,6 +144,17 @@ class ImouSensor(ImouEntity):
                 raise InvalidResponse(f"callbackUrl not found in {data}")
             self._state = data["callbackUrl"]
 
+        # status sensor
+        if self._name == "status":
+            # get the device status
+            data = await self.api_client.async_api_deviceOnline(self._device_id)
+            if "onLine" not in data:
+                raise InvalidResponse(f"onLine not found in {data}")
+            if data["onLine"] in ONLINE_STATUS:
+                self._state = ONLINE_STATUS[data["onLine"]]
+            else:
+                self._state = ONLINE_STATUS["UNKNOWN"]
+
         _LOGGER.debug(
             "[%s] updating %s, value is %s %s",
             self._device_name,
@@ -160,16 +195,15 @@ class ImouBinarySensor(ImouEntity):
 
     async def async_update(self, **kwargs):
         """Update the entity."""
-        if not self._enabled:
+        if not await self._async_is_ready():
             return
 
         # online sensor
         if self._name == "online":
-            # get the online status
-            data = await self.api_client.async_api_deviceOnline(self._device_id)
-            if "onLine" not in data:
-                raise InvalidResponse(f"onLine not found in {data}")
-            self._state = data["onLine"] == "1"
+            # get the device status
+            if self._device_instance is not None:
+                await self._device_instance.async_refresh_status()
+                self._state = self._device_instance.is_online()
 
         # motionAlarm sensor
         if self._name == "motionAlarm":
@@ -234,11 +268,13 @@ class ImouSwitch(ImouEntity):
 
     async def async_update(self, **kwargs):
         """Update the entity."""
-        if not self._enabled:
+        if not await self._async_is_ready():
             return
+
         # pushNotifications sensor
         if self._name == "pushNotifications":
             data = await self.api_client.async_api_getMessageCallback()
+
         # all the other dynamically created sensors
         else:
             data = await self.api_client.async_api_getDeviceCameraStatus(self._device_id, self._name)
@@ -259,8 +295,9 @@ class ImouSwitch(ImouEntity):
 
     async def async_turn_on(self, **kwargs):
         """Turn the entity on."""
-        if not self._enabled:
+        if not await self._async_is_ready():
             return
+
         _LOGGER.debug("[%s] %s requested to turn ON (%s)", self._device_name, self._description, kwargs)
         # pushNotifications sensor
         if self._name == "pushNotifications":
@@ -274,8 +311,9 @@ class ImouSwitch(ImouEntity):
 
     async def async_turn_off(self, **kwargs):
         """Turn the entity off."""
-        if not self._enabled:
+        if not await self._async_is_ready():
             return
+
         _LOGGER.debug("[%s] %s requested to turn OFF (%s)", self._device_name, self._description, kwargs)
         if self._name == "pushNotifications":
             await self.api_client.async_api_setMessageCallbackOff()
@@ -286,8 +324,9 @@ class ImouSwitch(ImouEntity):
 
     async def async_toggle(self, **kwargs):
         """Toggle the entity."""
-        if not self._enabled or not self._updated:
+        if not await self._async_is_ready():
             return
+
         if self._state:
             await self.async_turn_off()
         else:
@@ -320,8 +359,9 @@ class ImouSelect(ImouEntity):
 
     async def async_update(self, **kwargs):
         """Update the entity."""
-        if not self._enabled:
+        if not await self._async_is_ready():
             return
+
         if self._name == "nightVisionMode":
             # get the night vision mode option selected
             data = await self.api_client.async_api_getNightVisionMode(self._device_id)
@@ -349,8 +389,9 @@ class ImouSelect(ImouEntity):
 
     async def async_select_option(self, option: str) -> None:
         """Change the selected option."""
-        if not self._enabled:
+        if not await self._async_is_ready():
             return
+
         _LOGGER.debug("[%s] %s setting to %s", self._device_name, self._description, option)
         if self._name == "nightVisionMode":
             await self.api_client.async_api_setNightVisionMode(self._device_id, option)
@@ -380,8 +421,9 @@ class ImouButton(ImouEntity):
 
     async def async_press(self) -> None:
         """Press action."""
-        if not self._enabled:
+        if not await self._async_is_ready():
             return
+
         if self._name == "restartDevice":
             # restart the device
             await self.api_client.async_api_restartDevice(self._device_id)
@@ -423,8 +465,9 @@ class ImouSiren(ImouEntity):
 
     async def async_update(self, **kwargs):
         """Update the entity."""
-        if not self._enabled:
+        if not await self._async_is_ready():
             return
+
         # siren sensor
         if self._name == "siren":
             # async_api_getDeviceCameraStatus() does not return the current state of the siren, do nothing here
@@ -436,7 +479,7 @@ class ImouSiren(ImouEntity):
 
     async def async_turn_on(self, **kwargs):
         """Turn the entity on."""
-        if not self._enabled:
+        if not await self._async_is_ready():
             return
         _LOGGER.debug("[%s] %s requested to turn ON (%s)", self._device_name, self._description, kwargs)
         # siren sensor
@@ -446,7 +489,7 @@ class ImouSiren(ImouEntity):
 
     async def async_turn_off(self, **kwargs):
         """Turn the entity off."""
-        if not self._enabled:
+        if not await self._async_is_ready():
             return
         _LOGGER.debug("[%s] %s requested to turn OFF (%s)", self._device_name, self._description, kwargs)
         # siren sensor
@@ -456,7 +499,7 @@ class ImouSiren(ImouEntity):
 
     async def async_toggle(self, **kwargs):
         """Toggle the entity."""
-        if not self._enabled or not self._updated:
+        if not await self._async_is_ready():
             return
         if self._state:
             await self.async_turn_off()
@@ -473,6 +516,7 @@ class ImouCamera(ImouEntity):
         device_id: str,
         device_name: str,
         sensor_type: str,
+        profile: str,
     ) -> None:
         """
         Initialize the instance.
@@ -485,12 +529,134 @@ class ImouCamera(ImouEntity):
         """
         super().__init__(api_client, device_id, device_name, sensor_type, CAMERAS[sensor_type])
         self._state = False
+        self._profile = profile
 
     async def async_update(self, **kwargs):
         """Update the entity."""
-        if not self._enabled:
+        if not await self._async_is_ready():
             return
 
+    async def async_get_image(self) -> Union[bytes, None]:
+        """Get image snapshot."""
+        if not await self._async_is_ready():
+            return None
+        _LOGGER.debug(
+            "[%s] requested an image snapshot",
+            self._device_name,
+        )
+        # request a snapshot and get the url
+        data = await self.api_client.async_api_setDeviceSnapEnhanced(self._device_id)
+        if "url" not in data:
+            raise InvalidResponse(f"url not found in {data}")
+        url = data["url"]
+        # wait for the image to be available
+        camera_wait_before_download = CAMERA_WAIT_BEFORE_DOWNLOAD
+        if self._device_instance is not None:
+            self._device_instance.get_camera_wait_before_download()
+        await asyncio.sleep(camera_wait_before_download)
+        # retrieve the image from the url
+        session = self.api_client.get_session()
+        if session is None:
+            raise NotConnected()
+        try:
+            response = await session.request("GET", url, timeout=self.api_client.get_timeout())
+            if response.status != 200:
+                raise InvalidResponse(f"status code {response.status}")
+            image = await response.read()
+        except Exception as exception:
+            raise InvalidResponse(f"unable to retrieve image from {url}: {exception}") from exception
+        return image
+
+    async def async_open_stream(self) -> None:
+        """Open a new stream."""
+        if not await self._async_is_ready():
+            return
+        _LOGGER.debug(
+            "[%s] opening a new live stream",
+            self._device_name,
+        )
+        # Create a device live broadcast address for profile
+        try:
+            data = await self.api_client.async_api_bindDeviceLive(self._device_id, self._profile)
+            if "streams" not in data or "hls" not in data["streams"][0] or "liveToken" not in data:
+                raise InvalidResponse(f"streams, hls or liveToken not found in {data}")
+        except APIError as exception:
+            # The video live already exists
+            if "LV1001" in exception.to_string():
+                pass
+            else:
+                raise APIError from exception
+
+    async def async_get_existing_stream(self) -> dict:
+        """Get existing streams if any and return a data structure with url and token."""
+        existing_stream = {
+            "url": None,
+            "token": None,
+        }
+        data = {}
+        # request existing live streaming information for the device
+        try:
+            data = await self.api_client.async_api_getLiveStreamInfo(self._device_id)
+        except APIError as exception:
+            # The video live does not exist
+            if "LV1002" in exception.to_string():
+                pass
+            else:
+                raise APIError from exception
+        # streams already available, find the right one
+        if "streams" in data:
+            for stream in data["streams"]:
+                if (
+                    "streamId" not in stream
+                    or "status" not in stream
+                    or "hls" not in stream
+                    or "liveToken" not in stream
+                ):
+                    raise InvalidResponse(f"streamId, status, liveToken, hls not found in {stream}")
+                # identify the right stream for this profile
+                if (
+                    (
+                        (self._profile == "HD" and stream["streamId"] == 0)
+                        or (self._profile == "SD" and stream["streamId"] == 1)
+                    )
+                    and stream["hls"].startswith("https://")
+                    and stream["status"] == "1"
+                ):
+                    existing_stream["url"] = stream["hls"]
+                    existing_stream["token"] = stream["liveToken"]
+                    break
+            if existing_stream["url"] is None or existing_stream["token"] is None:
+                raise InvalidResponse(f"stream not found in {data}")
+        # return a data structure containing the url and the token
+        return existing_stream
+
+    async def async_close_stream(self) -> None:
+        """Close a live stream."""
+        # get the existing stream if any
+        existing_stream = await self.async_get_existing_stream()
+        if existing_stream["token"] is not None:
+            await self.api_client.async_api_unbindLive(existing_stream["token"])
+            _LOGGER.debug(
+                "[%s] closing live stream",
+                self._device_name,
+            )
+
+    async def async_get_stream_url(self) -> dict:
+        """Get a live stream URL, taking care of creating a stream if needed."""
+        # get the existing stream if any
+        existing_stream = await self.async_get_existing_stream()
+        if existing_stream["url"] is not None:
+            return existing_stream["url"]
+        else:
+            # otherwise open the stream
+            await self.async_open_stream()
+            # get the right stream url and return it
+            existing_stream = await self.async_get_existing_stream()
+            if existing_stream["url"] is None:
+                raise APIError("unable to get live streaming")
+            _LOGGER.debug("[%s] live streaming url: %s", self._device_name, existing_stream["url"])
+            return existing_stream["url"]
+
     async def async_service_ptz_location(self, horizontal: float, vertical: float, zoom: float) -> dict:
         """Perform PTZ location action."""
         _LOGGER.debug(
